
%
% The Filterred White Gaussian Noise Complex Gain Generator
%

%
% Constructor, performs initilization.
% The function has the following arguments:
%   blockSize: number of samples processed at one time, also determines the window size used when
%              truncating the impulse response of the coloring filter
%   fDts: normalized sample spacing
%   Kfactor: the K factor used for ricean fading
%   randomSeed: the random Seed (optional)
%   beta: the beta parameter for the Kaiser window
%
function c = FWNGen( varargin )

if nargin == 4 | nargin == 5

    % get arguments
    %
    blockSize = varargin{1};
    fDts = varargin{2};
    Kfactor = varargin{3};
    
    if nargin == 5
        randomSeed = varargin{4};
        beta = varargin{5}; 
    else
        % no random seed supplied
        randomSeed = sum(100*clock);        
        beta = varargin{4}; 
    end;
    
    %
    % Initialize
    %
    
    fprintf( 'generating time domain coloring filter...\n' );
    fprintf( '0%%-------50%%-------100%%\n' );
    
    % get the truncated time domain coloring filter 
    for i=[1:blockSize]
        timeFilter( 1, i ) = quad( @coloringfilter, 0, pi/2, [], [], (i-(blockSize-1)/2)*fDts );
        
        % print progress indicator
        if mod((i/blockSize), .05) == 0 
            fprintf( '>' );
        end;
    end;
    
    fprintf( '\n' );
    
    % apply the Kaiser window
    timeFilter = timeFilter .* rot90(kaiser( blockSize, beta ));
    
    % normalize to unit sum of squares
    sumSquared = sqrt( sum( abs(timeFilter).^2 ) );
    dummy = timeFilter / sumSquared;
    timeFilter = dummy;
    
    % Get the FIR filter in frequency domain
    % Need to pad with zeros to make a block twice the size
    c.filter = fft( [ timeFilter zeros( 1, blockSize ) ] );
    
    % initialize postCursor (used in the overlap-and-add method)
    % to zero, so that it can be used the first time generate 
    % is called.
    c.postCursor = zeros( 1, blockSize );
  
else
    if nargin == 1
        %
        % Create an object using the same properties as the object
        % passed in
        % NOTE: the random seed is not copied into the new object.
        %
        
        % source object
        source = varargin{1};
        
        % get base class properties
        blockSize = get( source, 'blockSize' );
        fDts = get( source, 'fDts' );
        Kfactor = get( source, 'Kfactor' );
        randomSeed = sum(100*clock);
        
        % copy the filter
        c.filter = source.filter;
        c.postCursor = zeros( 1, blockSize );
    else
        error( 'Wrong number of input arguments' );
    end
end;

% create base class
gainGen = ComplexGainGen( blockSize, fDts, Kfactor, randomSeed );

% create this class
c = class( c, 'FWNGen', gainGen );        



    

